@@ -13,12 +13,17 @@
 #include <stdlib.h>
 #include <time.h>
 
-#include <GL/glut.h>
-
 #ifdef WIN32
 #include <windows.h>
 #endif
 
+#include <GL/glut.h>
+
+#ifdef XMESA
+#include "GL/xmesa.h"
+static int fullscreen=1;
+#endif
+
 #ifndef M_PI
 #define M_PI 3.14159265
 #endif
@@ -111,7 +116,8 @@ static void reshape( int width, int height )
   glViewport(0, 0, (GLint)width, (GLint)height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
-  gluPerspective(50.0, ((GLfloat) width/(GLfloat)height), 1, lenghtXmnt*stepYmnt*0.7);
+  gluPerspective(50.0, ((GLfloat) width/(GLfloat)height), lenghtXmnt*stepYmnt*0.01,
+		 lenghtXmnt*stepYmnt*0.7);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
 }
@@ -379,7 +385,7 @@ void drawscene(void)
   glRasterPos2i(10,10);
   printstring(GLUT_BITMAP_HELVETICA_18,frbuf);
   glRasterPos2i(350,470);
-  printstring(GLUT_BITMAP_HELVETICA_10,"Terrain V1.1 Written by David Bucciarelli (tech.hmw@plus.it)");
+  printstring(GLUT_BITMAP_HELVETICA_10,"Terrain V1.2 Written by David Bucciarelli (tech.hmw@plus.it)");
   glRasterPos2i(434,457);
   printstring(GLUT_BITMAP_HELVETICA_10,"Based on a Mickael's demo (Skizo@Hol.Fr)");
 
@@ -435,6 +441,12 @@ static void key(unsigned char k, int x, int y)
       bfcull=1;
     }
     break;
+#ifdef XMESA
+  case ' ':
+    XMesaSetFXmode(fullscreen ? XMESA_FX_FULLSCREEN : XMESA_FX_WINDOW);
+    fullscreen=(!fullscreen);
+    break;
+#endif
   }
 }
 
@@ -531,7 +543,7 @@ static void loadpic (void)
   glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
   glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
 
-  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_LINEAR);
+  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
   glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 
   glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
@@ -574,7 +586,7 @@ int main(int ac, char **av)
 
   if(!(win=glutCreateWindow("Terrain"))) {
     fprintf(stderr,"Error, couldn't open window\n");
-    exit(-1);
+	return -1;
   }
 
   ModZMnt = 0.0f;
@@ -595,5 +607,5 @@ int main(int ac, char **av)
 
   glutMainLoop();
 
-  exit(0);
+  return 0;
 }
